import {
  clipboard,
  Form,
  expose,
  ui,
  TemplateUiCommand,
  kv,
  toast,
} from "@kksh/api/ui/template";
import { IMGUR_CLIENT_ID_KEY, URL_FORMAT_KEY, URL_FORMATS } from "./constants";

class ImgurExtPreference extends TemplateUiCommand {
  async load() {
    const urlFormat = await kv.get(URL_FORMAT_KEY);
    const clientId = await kv.get(IMGUR_CLIENT_ID_KEY);
    ui.render(
      new Form.Form({
        key: "imgur-upload-preferences",
        fields: [
          new Form.InputField({
            key: "clientId",
            default: clientId ?? undefined,
            label: "Client ID",
            placeholder: "Enter your Imgur client ID",
          }),
          new Form.SelectField({
            key: "urlFormat",
            options: URL_FORMATS,
            default: urlFormat ?? "plain",
            label: "URL Format",
          }),
        ],
      })
    );
  }

  onFormSubmit(formData: {
    clientId: string;
    urlFormat: string;
  }): Promise<void> {
    if (!formData.clientId) {
      toast.error("Client ID is required");
      return Promise.resolve();
    }

    return Promise.all([
      kv.set(IMGUR_CLIENT_ID_KEY, formData.clientId).catch((err) => {
        toast.error("Failed to save Client ID", {
          description: err.message,
        });
        throw err;
      }),
      kv.set(URL_FORMAT_KEY, formData.urlFormat).catch((err) => {
        toast.error("Failed to save URL Format", {
          description: err.message,
        });
        throw err;
      }),
    ]).then(() => {
      toast.success("Saved", {
        description: "You can now use the Imgur Upload Command",
      });
    });
  }
}

expose(new ImgurExtPreference());
